package acac;


public class MeetLOG implements Comparable<MeetLOG>, Cloneable
{
	private int id;
	private double out;
	private double in;
	private double beRejected;
	private double trust;
	private double trustIn;
	private double trustOut;
	
	public static final int always_cooperat_mode = 0; 
	public static final int defect_constant_mode = 1; 
	public static final int defect_linear_mode = 2; 
	public static final int defect_log_mode = 3; 
	
	private static final boolean IS_LOG_TRUST_INCREASE = true;
	private static final boolean IS_LINEAR_TRUST_INCREASE = false;
	private static final boolean IS_LOG_WANING = false;
	private static final boolean IS_LINEAR_WANING = true;
	private static final boolean IS_LOG_TRUST_DECREASE = true;
	private static final boolean IS_LINEAR_TRUST_DECREASE = false;
	
	
	private static final double MAX_COMPRESS_RATIO = 10.0d;// weak ties preferred
	private static final double MIN_COMPRESS_RATIO = 0.0d;
	public static final double MAX_TRUST = 100;
	private static final double COMPRESSION_INTERVAL = 
		(MAX_COMPRESS_RATIO - MIN_COMPRESS_RATIO) / MAX_TRUST;
	private static final double MIN_WARNING = 8.0d;
	private static final double WARNING_C = 0.001d;
	
	private double rejectProbability;
	private static double maxDefectRejectStep = 0.1; 
	private static double minDefectRejectStep = 0; 
	private static double defectRejectStep = 0.1; 
	

	private static final double RISK_RATE_CONS = 0.02d;
	
	private static final double TRUST_INCREASE_STEP_LINEARLY = 10;//40.0d for linear increase-log waning;
	private static final double TRUST_DECREASE_STEP_LINEARLY = 1.0d;
	private static final double TRUST_WANINE_STEP_LINEARLY = 0.08d;//weak ties preferred
	
	MeetLOG(int id)
	{
		this.id = id;
		in = 0;
		out = 0;
		trust = 0.0d;
		trustIn = 0.0d;
		trustOut = 0.0d;
		beRejected = 0.0d;
		rejectProbability = 0;
	}
	
	boolean Equals(Object o)
	{
		if(o instanceof MeetLOG && ((MeetLOG)o).getID() == id)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public int getID()
	{
		return id;	
	}
	
	public void increaseTrust()
	{
		if(IS_LOG_TRUST_INCREASE)
		{
			trust = trust + Math.max(0,
				MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trust);
		}
		else if(IS_LINEAR_TRUST_INCREASE)
		{
			trust = trust + TRUST_INCREASE_STEP_LINEARLY;
		}	
		else
		{
			
		}
	}
	
	public void increaseTrustIn()
	{
		if(IS_LOG_TRUST_INCREASE)
		{
			trustIn = trustIn + Math.max(0, 
				MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trustIn);
		}
		else if(IS_LINEAR_TRUST_INCREASE)
		{
			trustIn = trustIn + TRUST_INCREASE_STEP_LINEARLY;
		}
		else
		{
			
		}
	}

	public void increaseTrustOut()
	{
		if(IS_LOG_TRUST_INCREASE)
		{
			trustOut = trustOut + Math.max(0, 
				MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trustOut);
		}
		else if(IS_LINEAR_TRUST_INCREASE)
		{
			trustOut = trustOut + TRUST_INCREASE_STEP_LINEARLY;
		}
		else
		{
			
		}
	}
	
	
	public void addIn()
	{
		in++;
	}

	public void addOut()
	{
		rejectProbability = rejectProbability - maxDefectRejectStep + 
					defectRejectStep * rejectProbability;
		out++;
	}
	
	public void decreaseTrust()
	{
		if(IS_LOG_TRUST_DECREASE)
		{
			trust = Math.max(trust - Math.max(0, 
				(MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trust)), 0);
		}
		else if(IS_LINEAR_TRUST_DECREASE)
		{
			trust = Math.max(0,trust - TRUST_DECREASE_STEP_LINEARLY);
		}
		else
		{
			
		}
	}
	public void decreaseTrustIn()
	{
		if(IS_LOG_TRUST_DECREASE)
		{
			trustIn = Math.max(trustIn - Math.max(0,
				MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trustIn) / 4, 0);
		}
		else if(IS_LINEAR_TRUST_DECREASE)
		{
			trustIn = Math.max(0,trustIn - TRUST_DECREASE_STEP_LINEARLY);
		}
		else
		{
			
		}

	}
	public void decreaseTrustOut()
	{
		if(IS_LOG_TRUST_DECREASE)
		{
			trustOut = Math.max(trustOut - Math.max(0,
				MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trustOut), 0);
		}
		else if(IS_LINEAR_TRUST_DECREASE)
		{
			trustOut = Math.max(0,trustOut - TRUST_DECREASE_STEP_LINEARLY);
		}
		else
		{
			
		}
	}
	
	public double getInteraction()
	{
		return in + out;
	}
	
	public double getIn()
	{
		return in;
	}
	
	public void setIn(double in)
	{
		this.in = in;
	}
	
	public double getOut()
	{
		return out;
	}

	public void setOut(double out)
	{
		this.out = out;
	}
	
	public double getTrust()
	{
		return trust;
	}

	public double getRisk()
	{
		return out * RISK_RATE_CONS;
	}
	
	public double getDefenseProb(int defeceMode, double defectRate)
	{
		if(defeceMode == always_cooperat_mode)
		{
			return 0;
		}
		else if(defeceMode == defect_constant_mode)
		{
			return defectRate;
		}
		else if(defeceMode == defect_linear_mode)
		{
			return beRejected / (out + beRejected);
		}
		else if(defeceMode == defect_log_mode)
		{
			return rejectProbability; 
		}
		else
		{
			return 0;
		}
		
	}
	
	public void increaseBeRejected()
	{
		rejectProbability = rejectProbability + maxDefectRejectStep - 
					defectRejectStep * rejectProbability;
		beRejected++;
	}
	
	public void setTrust(double trust)
	{
		this.trust = trust;
	}

	public void setTrustIn(double trustIn)
	{
		this.trustIn = trustIn;
	}

	public void setTrustOut(double trustOut)
	{
		this.trustOut = trustOut;
	}
	
	public void waning()
	{
		if(IS_LOG_WANING)
		{
			trust = Math.max(0, trust - Math.max(0,
					MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trust));
			trustIn = Math.max(0, trustIn - Math.max(0,
					MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trustIn));
			trustOut = Math.max(0, trustOut - Math.max(0,
					MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trustOut));
//			trust = Math.max(0, trust - WARNING_C * (1 + MIN_WARNING - trust / MAX_TRUST));
//			trustIn = Math.max(0, trustIn - WARNING_C * (1 + MIN_WARNING - trustIn / MAX_TRUST));
//			trustOut = Math.max(0, trustOut - WARNING_C * (1 + MIN_WARNING - trustOut / MAX_TRUST));
		}
		else if(IS_LINEAR_WANING)
		{
			trust = Math.max(0, trust - TRUST_WANINE_STEP_LINEARLY);
			trustIn = Math.max(0, trustIn - TRUST_WANINE_STEP_LINEARLY);
			trustOut = Math.max(0, trustOut - TRUST_WANINE_STEP_LINEARLY);
		}
		else
		{
			
		}		
	}
	
	 public int compareTo(MeetLOG o) 
	 throws ClassCastException 
	 {
		 double comparedTrust = o.getTrust();
		 if(trust > comparedTrust)
		 {
			 return 1;
		 }
		 else if(trust < comparedTrust)
		 {
			 return -1;
		 }
		 else
		 {
			 return 0;
		 }
	 }
}
